package elpanel;

import com.sonycsl.echo.Echo;
import com.sonycsl.echo.EchoProperty;
import com.sonycsl.echo.eoj.EchoObject;
import com.sonycsl.echo.eoj.device.DeviceObject;
import com.sonycsl.echo.eoj.device.housingfacilities.GeneralLighting;
import com.sonycsl.echo.eoj.profile.NodeProfile;
import com.sonycsl.echo.processing.defaults.DefaultController;
import com.sonycsl.echo.processing.defaults.DefaultNodeProfile;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.application.Application;
import static javafx.application.Platform.exit;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;

public class Elpanel extends Application {
    private static GeneralLighting mArmadilloLighting;

    @Override
    public void start(Stage primaryStage) throws IOException {
        Parent root = FXMLLoader.load(getClass().getResource("Elpanel.fxml"));
        Scene scene = new Scene(root);

        primaryStage.setTitle("ECHONET Lite controller");
        primaryStage.setScene(scene);
        primaryStage.show();

        Echo.start(new DefaultNodeProfile(),
                   new DeviceObject[]{new DefaultController()});
        /* 機器リストアップのためのECHONET Liteブロードキャスト送信 */
        NodeProfile.getG().reqGetSelfNodeInstanceListS().send();
    }

    public static void main(String[] args) {
        Echo.addEventListener(new Echo.EventListener() {
            @Override
            public void onNewGeneralLighting(GeneralLighting device) {
                super.onNewGeneralLighting(device);
                device.setReceiver(new GeneralLighting.Receiver() {
                    @Override
                    protected void onGetManufacturerCode
                            (EchoObject eoj, short tid, byte esv,
                             EchoProperty property, boolean success) {
                        super.onGetManufacturerCode(eoj, tid, esv,
                                                    property, success);
                        /* 反応した一般照明のリストアップ */
                        switch (property.edt[2]) {
                        case 0x00:
                            /* 未定義=Armadillo */
                            mArmadilloLighting = device;
                            break;
                        }
                        try {
                            /* 機器の状態取得のためECHNET Lite送信 */
                            device.get().reqGetOperationStatus().send();
                        } catch (IOException ex) {
                            Logger.getLogger(Elpanel.class.getName()).log
                                    (Level.SEVERE, null, ex);
                        }
                    }
                    @Override
                    protected void onGetOperationStatus
                            (EchoObject eoj, short tid, byte esv,
                             EchoProperty property, boolean success) {
                        super.onGetOperationStatus(eoj, tid, esv,
                                                   property, success);
                        try {
                            /* 機器ON/OFF状態をElpanelController通知 */
                            if (device == mArmadilloLighting) {
                                /* Armadillo照明 */
                                switch (property.edt[0]) {
                                case 0x30:
                                    sendMessage(1, true);
                                    break;
                                case 0x31:
                                    sendMessage(1, false);
                                    break;
                                }
                            }
                        } catch (IOException ex) {
                            Logger.getLogger(Elpanel.class.getName()).log
                                (Level.SEVERE, null, ex);
                        }
                    }
                });
                try {
                    device.get().reqGetManufacturerCode().send();
                } catch (IOException ex) {
                    Logger.getLogger(Elpanel.class.getName()).log
                        (Level.SEVERE, null, ex);
                }
            }
        });

        launch(args);
    }

    /* Armadillo照明へのECHONET Lite制御 */
    public static void ArmadilloLighting(boolean on) {
        try {
            if (mArmadilloLighting != null)
                mArmadilloLighting.set().reqSetOperationStatus
                        (new byte[]{on ? (byte)0x30: (byte)0x31}).send();
        } catch (IOException ex) {
            Logger.getLogger(Elpanel.class.getName()).log(Level.SEVERE,
                                                          null, ex);
        }
    }

    /* ElpanelControllerへの機器ON/OFF通知 */
    public static void sendMessage(int id, boolean on) throws IOException {
        InetSocketAddress socketAddress =
                new InetSocketAddress("localhost", ElpanelController.port);
        Socket socket = new Socket();
        socket.connect(socketAddress);
        String message = id + ": " + (on ? "on" : "off");
        PrintWriter writer = new PrintWriter(socket.getOutputStream(), true);
        writer.println(message);
        writer.flush();
    }
}
